@@ -1,5 +1,6 @@
 /*
-** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2006 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2005 David Viens <davidv@plogue.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,7 +17,7 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include "config.h"
+#include "sfconfig.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,52 +33,53 @@
  * Macros to handle big/little endian issues.
  */
 
-#define FORM_MARKER	(MAKE_MARKER ('F', 'O', 'R', 'M'))
-#define AIFF_MARKER	(MAKE_MARKER ('A', 'I', 'F', 'F'))
-#define AIFC_MARKER	(MAKE_MARKER ('A', 'I', 'F', 'C'))
-#define COMM_MARKER	(MAKE_MARKER ('C', 'O', 'M', 'M'))
-#define SSND_MARKER	(MAKE_MARKER ('S', 'S', 'N', 'D'))
-#define MARK_MARKER	(MAKE_MARKER ('M', 'A', 'R', 'K'))
-#define INST_MARKER	(MAKE_MARKER ('I', 'N', 'S', 'T'))
-#define APPL_MARKER	(MAKE_MARKER ('A', 'P', 'P', 'L'))
+#define FORM_MARKER		(MAKE_MARKER ('F', 'O', 'R', 'M'))
+#define AIFF_MARKER		(MAKE_MARKER ('A', 'I', 'F', 'F'))
+#define AIFC_MARKER		(MAKE_MARKER ('A', 'I', 'F', 'C'))
+#define COMM_MARKER		(MAKE_MARKER ('C', 'O', 'M', 'M'))
+#define SSND_MARKER		(MAKE_MARKER ('S', 'S', 'N', 'D'))
+#define MARK_MARKER		(MAKE_MARKER ('M', 'A', 'R', 'K'))
+#define INST_MARKER		(MAKE_MARKER ('I', 'N', 'S', 'T'))
+#define APPL_MARKER		(MAKE_MARKER ('A', 'P', 'P', 'L'))
 
-#define c_MARKER	(MAKE_MARKER ('(', 'c', ')', ' '))
-#define NAME_MARKER	(MAKE_MARKER ('N', 'A', 'M', 'E'))
-#define AUTH_MARKER	(MAKE_MARKER ('A', 'U', 'T', 'H'))
-#define ANNO_MARKER	(MAKE_MARKER ('A', 'N', 'N', 'O'))
-#define COMT_MARKER	(MAKE_MARKER ('C', 'O', 'M', 'T'))
-#define FVER_MARKER	(MAKE_MARKER ('F', 'V', 'E', 'R'))
-#define SFX_MARKER	(MAKE_MARKER ('S', 'F', 'X', '!'))
+#define c_MARKER		(MAKE_MARKER ('(', 'c', ')', ' '))
+#define NAME_MARKER		(MAKE_MARKER ('N', 'A', 'M', 'E'))
+#define AUTH_MARKER		(MAKE_MARKER ('A', 'U', 'T', 'H'))
+#define ANNO_MARKER		(MAKE_MARKER ('A', 'N', 'N', 'O'))
+#define COMT_MARKER		(MAKE_MARKER ('C', 'O', 'M', 'T'))
+#define FVER_MARKER		(MAKE_MARKER ('F', 'V', 'E', 'R'))
+#define SFX_MARKER		(MAKE_MARKER ('S', 'F', 'X', '!'))
 
-#define PEAK_MARKER	(MAKE_MARKER ('P', 'E', 'A', 'K'))
+#define PEAK_MARKER		(MAKE_MARKER ('P', 'E', 'A', 'K'))
+#define basc_MARKER		(MAKE_MARKER ('b', 'a', 's', 'c'))
 
 /* Supported AIFC encodings.*/
-#define NONE_MARKER	(MAKE_MARKER ('N', 'O', 'N', 'E'))
-#define sowt_MARKER	(MAKE_MARKER ('s', 'o', 'w', 't'))
-#define twos_MARKER	(MAKE_MARKER ('t', 'w', 'o', 's'))
-#define raw_MARKER	(MAKE_MARKER ('r', 'a', 'w', ' '))
-#define in32_MARKER	(MAKE_MARKER ('i', 'n', '3', '2'))
-#define ni32_MARKER	(MAKE_MARKER ('2', '3', 'n', 'i'))
+#define NONE_MARKER		(MAKE_MARKER ('N', 'O', 'N', 'E'))
+#define sowt_MARKER		(MAKE_MARKER ('s', 'o', 'w', 't'))
+#define twos_MARKER		(MAKE_MARKER ('t', 'w', 'o', 's'))
+#define raw_MARKER		(MAKE_MARKER ('r', 'a', 'w', ' '))
+#define in32_MARKER		(MAKE_MARKER ('i', 'n', '3', '2'))
+#define ni32_MARKER		(MAKE_MARKER ('2', '3', 'n', 'i'))
 
-#define fl32_MARKER	(MAKE_MARKER ('f', 'l', '3', '2'))
-#define FL32_MARKER	(MAKE_MARKER ('F', 'L', '3', '2'))
-#define fl64_MARKER	(MAKE_MARKER ('f', 'l', '6', '4'))
-#define FL64_MARKER	(MAKE_MARKER ('F', 'L', '6', '4'))
+#define fl32_MARKER		(MAKE_MARKER ('f', 'l', '3', '2'))
+#define FL32_MARKER		(MAKE_MARKER ('F', 'L', '3', '2'))
+#define fl64_MARKER		(MAKE_MARKER ('f', 'l', '6', '4'))
+#define FL64_MARKER		(MAKE_MARKER ('F', 'L', '6', '4'))
 
-#define ulaw_MARKER	(MAKE_MARKER ('u', 'l', 'a', 'w'))
-#define ULAW_MARKER	(MAKE_MARKER ('U', 'L', 'A', 'W'))
-#define alaw_MARKER	(MAKE_MARKER ('a', 'l', 'a', 'w'))
-#define ALAW_MARKER	(MAKE_MARKER ('A', 'L', 'A', 'W'))
+#define ulaw_MARKER		(MAKE_MARKER ('u', 'l', 'a', 'w'))
+#define ULAW_MARKER		(MAKE_MARKER ('U', 'L', 'A', 'W'))
+#define alaw_MARKER		(MAKE_MARKER ('a', 'l', 'a', 'w'))
+#define ALAW_MARKER		(MAKE_MARKER ('A', 'L', 'A', 'W'))
 
-#define DWVW_MARKER	(MAKE_MARKER ('D', 'W', 'V', 'W'))
-#define GSM_MARKER	(MAKE_MARKER ('G', 'S', 'M', ' '))
-#define ima4_MARKER	(MAKE_MARKER ('i', 'm', 'a', '4'))
+#define DWVW_MARKER		(MAKE_MARKER ('D', 'W', 'V', 'W'))
+#define GSM_MARKER		(MAKE_MARKER ('G', 'S', 'M', ' '))
+#define ima4_MARKER		(MAKE_MARKER ('i', 'm', 'a', '4'))
 
 /* Unsupported AIFC encodings.*/
 
-#define MAC3_MARKER	(MAKE_MARKER ('M', 'A', 'C', '3'))
-#define MAC6_MARKER	(MAKE_MARKER ('M', 'A', 'C', '6'))
-#define ADP4_MARKER	(MAKE_MARKER ('A', 'D', 'P', '4'))
+#define MAC3_MARKER		(MAKE_MARKER ('M', 'A', 'C', '3'))
+#define MAC6_MARKER		(MAKE_MARKER ('M', 'A', 'C', '6'))
+#define ADP4_MARKER		(MAKE_MARKER ('A', 'D', 'P', '4'))
 
 /* Predfined chunk sizes. */
 #define SIZEOF_AIFF_COMM		18
@@ -86,19 +88,23 @@
 #define SIZEOF_SSND_CHUNK		8
 #define SIZEOF_INST_CHUNK		20
 
+/* Is it constant? */
+
 /* AIFC/IMA4 defines. */
 #define AIFC_IMA4_BLOCK_LEN				34
-#define	AIFC_IMA4_SAMPLES_PER_BLOCK		64
+#define AIFC_IMA4_SAMPLES_PER_BLOCK		64
+
+#define AIFF_PEAK_CHUNK_SIZE(ch)	(2 * sizeof (int) + ch * (sizeof (float) + sizeof (int)))
 
 /*------------------------------------------------------------------------------
  * Typedefs for file chunks.
  */
 
 enum
-{	HAVE_FORM	= 0x01,
-	HAVE_AIFF	= 0x02,
-	HAVE_COMM	= 0x04,
-	HAVE_SSND	= 0x08
+{	HAVE_FORM		= 0x01,
+	HAVE_AIFF		= 0x02,
+	HAVE_COMM		= 0x04,
+	HAVE_SSND		= 0x08
 } ;
 
 typedef struct
@@ -118,7 +124,7 @@ typedef struct
 
 typedef struct
 {	short			playMode ;
-    unsigned short	beginLoop ;
+	unsigned short	beginLoop ;
 	unsigned short	endLoop ;
 } INST_LOOP ;
 
@@ -134,14 +140,43 @@ typedef struct
 	INST_LOOP	release_loop ;
 } INST_CHUNK ;
 
+
+enum
+{	basc_SCALE_MINOR = 1,
+	basc_SCALE_MAJOR,
+	basc_SCALE_NEITHER,
+	basc_SCALE_BOTH
+} ;
+
+enum
+{	basc_TYPE_LOOP = 0,
+	basc_TYPE_ONE_SHOT
+} ;
+
+
+typedef struct
+{	unsigned int	version ;
+	unsigned int	numBeats ;
+	unsigned short	rootNote ;
+	unsigned short	scaleType ;
+	unsigned short	sigNumerator ;
+	unsigned short	sigDenominator ;
+	unsigned short	loopType ;
+} basc_CHUNK ;
+
+typedef struct
+{	unsigned short	markerID ;
+	unsigned int	position ;
+} MARK_ID_POS ;
+
 /*------------------------------------------------------------------------------
  * Private static functions.
  */
 
-static int	aiff_close	(SF_PRIVATE *psf) ;
+static int	aiff_close (SF_PRIVATE *psf) ;
 
 static int	tenbytefloat2int (unsigned char *bytes) ;
-static void	uint2tenbytefloat (unsigned int num, unsigned char *bytes) ;
+static void uint2tenbytefloat (unsigned int num, unsigned char *bytes) ;
 
 static int	aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt) ;
 
@@ -149,22 +184,29 @@ static int	aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt) ;
 
 static int	aiff_write_header (SF_PRIVATE *psf, int calc_length) ;
 static int	aiff_write_tailer (SF_PRIVATE *psf) ;
-static void aiff_write_strings (SF_PRIVATE *psf, int location) ;
+static void	aiff_write_strings (SF_PRIVATE *psf, int location) ;
 
 static int	aiff_command (SF_PRIVATE *psf, int command, void *data, int datasize) ;
 
 static const char *get_loop_mode_str (short mode) ;
 
+static short get_loop_mode (short mode) ;
+
+static int aiff_read_basc_chunk (SF_PRIVATE * psf, int) ;
+
+static unsigned int marker_to_position (const MARK_ID_POS *m, unsigned short n, int marksize) ;
 
 /*------------------------------------------------------------------------------
 ** Public function.
 */
 
 int
-aiff_open	(SF_PRIVATE *psf)
+aiff_open (SF_PRIVATE *psf)
 {	COMM_CHUNK comm_fmt ;
 	int error, subformat ;
 
+	memset (&comm_fmt, 0, sizeof (comm_fmt)) ;
+
 	subformat = psf->sf.format & SF_FORMAT_SUBMASK ;
 
 	if (psf->mode == SFM_READ || (psf->mode == SFM_RDWR && psf->filelength > 0))
@@ -179,14 +221,12 @@ aiff_open	(SF_PRIVATE *psf)
 			return SFE_NO_PIPE_WRITE ;
 
 		if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_AIFF)
-			return	SFE_BAD_OPEN_FORMAT ;
+			return SFE_BAD_OPEN_FORMAT ;
 
 		if (psf->mode == SFM_WRITE && (subformat == SF_FORMAT_FLOAT || subformat == SF_FORMAT_DOUBLE))
-		{	psf->pchunk = calloc (1, sizeof (PEAK_CHUNK) * psf->sf.channels * sizeof (PEAK_POS)) ;
-			if (psf->pchunk == NULL)
+		{	if ((psf->peak_info = peak_info_calloc (psf->sf.channels)) == NULL)
 				return SFE_MALLOC_FAILED ;
-			psf->has_peak = SF_TRUE ;
-			psf->peak_loc = SF_PEAK_START ;
+			psf->peak_info->peak_loc = SF_PEAK_START ;
 			} ;
 
 		if (psf->mode != SFM_RDWR || psf->filelength < 40)
@@ -204,11 +244,9 @@ aiff_open	(SF_PRIVATE *psf)
 		psf->write_header = aiff_write_header ;
 		} ;
 
-	psf->close = aiff_close ;
+	psf->container_close = aiff_close ;
 	psf->command = aiff_command ;
 
-	psf->blockwidth = psf->sf.channels * psf->bytewidth ;
-
 	switch (psf->sf.format & SF_FORMAT_SUBMASK)
 	{	case SF_FORMAT_PCM_U8 :
 				error = pcm_init (psf) ;
@@ -283,8 +321,6 @@ aiff_open	(SF_PRIVATE *psf)
 		default : return SFE_UNIMPLEMENTED ;
 		} ;
 
-	if (psf->mode == SFM_READ)
-		psf->blockwidth = psf->sf.channels * psf->bytewidth ;
 
 	return error ;
 } /* aiff_open */
@@ -293,13 +329,25 @@ aiff_open	(SF_PRIVATE *psf)
 ** Private functions.
 */
 
+/* This function ought to check size */
+static unsigned int
+marker_to_position (const MARK_ID_POS *m, unsigned short n, int marksize)
+{	int i ;
+
+    for (i = 0 ; i < marksize ; i++)
+		if (m [i].markerID == n)
+			return m [i].position ;
+    return 0 ;
+} /* marker_to_position */
+
 static int
 aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 {	SSND_CHUNK	ssnd_fmt ;
-	int			marker, dword, bytesread, k ;
-	int			FORMsize, SSNDsize ;
-	int			filetype, found_chunk = 0, done = 0, error = 0 ;
+	MARK_ID_POS *markstr = NULL ;
+	unsigned	marker, dword, FORMsize, SSNDsize, bytesread ;
+	int			k, found_chunk = 0, done = 0, error = 0 ;
 	char		*cptr, byte ;
+	int			instr_found = 0, mark_found = 0, mark_count = 0 ;
 
 	/* Set position to start of file to begin reading header. */
 	psf_binheader_readf (psf, "p", 0) ;
@@ -309,9 +357,9 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 	/* Until recently AIF* file were all BIG endian. */
 	psf->endian = SF_ENDIAN_BIG ;
 
-	/* 	AIFF files can apparently have their chunks in any order. However, they
+	/*	AIFF files can apparently have their chunks in any order. However, they
 	**	must have a FORM chunk. Approach here is to read all the chunks one by
-	**  one and then check for the mandatory chunks at the end.
+	**	one and then check for the mandatory chunks at the end.
 	*/
 	while (! done)
 	{	psf_binheader_readf (psf, "m", &marker) ;
@@ -343,9 +391,8 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 
 			case AIFC_MARKER :
 			case AIFF_MARKER :
-					if (! (found_chunk & HAVE_FORM))
+					if ((found_chunk & HAVE_FORM) == 0)
 						return SFE_AIFF_AIFF_NO_FORM ;
-					filetype = marker ;
 					psf_log_printf (psf, " %M\n", marker) ;
 					found_chunk |= HAVE_AIFF ;
 					break ;
@@ -353,10 +400,10 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 			case COMM_MARKER :
 					error = aiff_read_comm_chunk (psf, comm_fmt) ;
 
-					psf->sf.samplerate 	= tenbytefloat2int (comm_fmt->sampleRate) ;
-					psf->sf.frames 	= comm_fmt->numSampleFrames ;
-					psf->sf.channels 	= comm_fmt->numChannels ;
-					psf->bytewidth 		= BITWIDTH2BYTES (comm_fmt->sampleSize) ;
+					psf->sf.samplerate = tenbytefloat2int (comm_fmt->sampleRate) ;
+					psf->sf.frames = comm_fmt->numSampleFrames ;
+					psf->sf.channels = comm_fmt->numChannels ;
+					psf->bytewidth = BITWIDTH2BYTES (comm_fmt->sampleSize) ;
 
 					if (error)
 						return error ;
@@ -372,39 +419,41 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 					psf_binheader_readf (psf, "E4", &dword) ;
 
 					psf_log_printf (psf, "%M : %d\n", marker, dword) ;
-					if (dword != SIGNED_SIZEOF (PEAK_CHUNK) + psf->sf.channels * SIGNED_SIZEOF (PEAK_POS))
+					if (dword != AIFF_PEAK_CHUNK_SIZE (psf->sf.channels))
 					{	psf_binheader_readf (psf, "j", dword) ;
-						psf_log_printf (psf, "*** File PEAK chunk bigger than sizeof (PEAK_CHUNK).\n") ;
+						psf_log_printf (psf, "*** File PEAK chunk too big.\n") ;
 						return SFE_WAV_BAD_PEAK ;
 						} ;
 
-					psf->pchunk = calloc (1, sizeof (PEAK_CHUNK) * psf->sf.channels * sizeof (PEAK_POS)) ;
-					if (psf->pchunk == NULL)
+					if ((psf->peak_info = peak_info_calloc (psf->sf.channels)) == NULL)
 						return SFE_MALLOC_FAILED ;
 
 					/* read in rest of PEAK chunk. */
-					psf_binheader_readf (psf, "E44", &(psf->pchunk->version), &(psf->pchunk->timestamp)) ;
+					psf_binheader_readf (psf, "E44", &(psf->peak_info->version), &(psf->peak_info->timestamp)) ;
 
-					if (psf->pchunk->version != 1)
-						psf_log_printf (psf, "  version    : %d *** (should be version 1)\n", psf->pchunk->version) ;
+					if (psf->peak_info->version != 1)
+						psf_log_printf (psf, "  version    : %d *** (should be version 1)\n", psf->peak_info->version) ;
 					else
-						psf_log_printf (psf, "  version    : %d\n", psf->pchunk->version) ;
+						psf_log_printf (psf, "  version    : %d\n", psf->peak_info->version) ;
 
-					psf_log_printf (psf, "  time stamp : %d\n", psf->pchunk->timestamp) ;
+					psf_log_printf (psf, "  time stamp : %d\n", psf->peak_info->timestamp) ;
 					psf_log_printf (psf, "    Ch   Position       Value\n") ;
 
-					cptr = (char *) psf->buffer ;
-					for (dword = 0 ; dword < psf->sf.channels ; dword++)
-					{	psf_binheader_readf (psf, "Ef4", &(psf->pchunk->peaks [dword].value),
-														&(psf->pchunk->peaks [dword].position)) ;
+					cptr = psf->u.cbuf ;
+					for (dword = 0 ; dword < (unsigned) psf->sf.channels ; dword++)
+					{	float value ;
+						unsigned int position ;
 
-						LSF_SNPRINTF (cptr, sizeof (psf->buffer), "    %2d   %-12d   %g\n",
-								dword, psf->pchunk->peaks [dword].position, psf->pchunk->peaks [dword].value) ;
-						cptr [sizeof (psf->buffer) - 1] = 0 ;
+						psf_binheader_readf (psf, "Ef4", &value, &position) ;
+						psf->peak_info->peaks [dword].value = value ;
+						psf->peak_info->peaks [dword].position = position ;
+
+						LSF_SNPRINTF (cptr, sizeof (psf->u.scbuf), "    %2d   %-12ld   %g\n",
+								dword, (long) psf->peak_info->peaks [dword].position, psf->peak_info->peaks [dword].value) ;
+						cptr [sizeof (psf->u.scbuf) - 1] = 0 ;
 						psf_log_printf (psf, cptr) ;
 						} ;
 
-					psf->has_peak = SF_TRUE ; /* Found PEAK chunk. */
 					break ;
 
 			case SSND_MARKER :
@@ -432,77 +481,88 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 					if (! psf->sf.seekable)
 						break ;
 
-					/* 	Seek to end of SSND chunk. */
+					/*      Seek to end of SSND chunk. */
 					psf_fseek (psf, psf->dataoffset + psf->datalength + (SSNDsize & 1), SEEK_SET) ;
 					break ;
 
 			case c_MARKER :
 					psf_binheader_readf (psf, "E4", &dword) ;
-					dword += (dword & 1) ;
 					if (dword == 0)
 						break ;
-					if (dword > SIGNED_SIZEOF (psf->buffer))
+					if (dword > SIGNED_SIZEOF (psf->u.scbuf) - 1)
 					{	psf_log_printf (psf, " %M : %d (too big)\n", marker, dword) ;
 						return SFE_INTERNAL ;
 						} ;
 
-					cptr = (char*) psf->buffer ;
-					psf_binheader_readf (psf, "b", cptr, dword) ;
-					cptr [dword - 1] = 0 ;
+					cptr = psf->u.cbuf ;
+					psf_binheader_readf (psf, "b", cptr, dword + (dword & 1)) ;
+					cptr [dword] = 0 ;
 					psf_log_printf (psf, " %M : %s\n", marker, cptr) ;
 					psf_store_string (psf, SF_STR_COPYRIGHT, cptr) ;
 					break ;
 
 			case AUTH_MARKER :
 					psf_binheader_readf (psf, "E4", &dword) ;
-					dword += (dword & 1) ;
 					if (dword == 0)
 						break ;
-					if (dword > SIGNED_SIZEOF (psf->buffer))
+					if (dword > SIGNED_SIZEOF (psf->u.scbuf) - 1)
 					{	psf_log_printf (psf, " %M : %d (too big)\n", marker, dword) ;
 						return SFE_INTERNAL ;
 						} ;
 
-					cptr = (char*) psf->buffer ;
-					psf_binheader_readf (psf, "b", cptr, dword) ;
-					cptr [dword - 1] = 0 ;
+					cptr = psf->u.cbuf ;
+					psf_binheader_readf (psf, "b", cptr, dword + (dword & 1)) ;
+					cptr [dword] = 0 ;
 					psf_log_printf (psf, " %M : %s\n", marker, cptr) ;
 					psf_store_string (psf, SF_STR_ARTIST, cptr) ;
 					break ;
 
 			case COMT_MARKER :
-					psf_binheader_readf (psf, "E4", &dword) ;
+				{	unsigned short count, id, len ;
+					unsigned int timestamp ;
+
+					psf_binheader_readf (psf, "E42", &dword, &count) ;
+					psf_log_printf (psf, " %M : %d\n  count  : %d\n", marker, dword, count) ;
 					dword += (dword & 1) ;
 					if (dword == 0)
 						break ;
-					if (dword > SIGNED_SIZEOF (psf->buffer))
-					{	psf_log_printf (psf, " %M : %d (too big)\n", marker, dword) ;
-						return SFE_INTERNAL ;
+					dword -= 2 ;
+
+					for (k = 0 ; k < count ; k++)
+					{	dword -= psf_binheader_readf (psf, "E422", &timestamp, &id, &len) ;
+						psf_log_printf (psf, "   time   : 0x%x\n   marker : %x\n   length : %d\n", timestamp, id, len) ;
+
+						if (len + 1 > SIGNED_SIZEOF (psf->u.scbuf))
+						{	psf_log_printf (psf, "\nError : string length (%d) too big.\n", len) ;
+							return SFE_INTERNAL ;
+							} ;
+
+						cptr = psf->u.cbuf ;
+						dword -= psf_binheader_readf (psf, "b", cptr, len) ;
+						cptr [len] = 0 ;
+						psf_log_printf (psf, "   string : %s\n", cptr) ;
 						} ;
 
-					cptr = (char*) psf->buffer ;
-					psf_binheader_readf (psf, "b", cptr, dword) ;
-					cptr [dword - 1] = 0 ;
-					psf_log_printf (psf, " %M : %s\n", marker, cptr) ;
-					psf_store_string (psf, SF_STR_COMMENT, cptr) ;
+					if (dword > 0)
+						psf_binheader_readf (psf, "j", dword) ;
+					} ;
 					break ;
 
 			case APPL_MARKER :
 					psf_binheader_readf (psf, "E4", &dword) ;
-					dword += (dword & 1) ;
 					if (dword == 0)
 						break ;
-					if (dword >= SIGNED_SIZEOF (psf->buffer))
+					if (dword >= SIGNED_SIZEOF (psf->u.scbuf) - 1)
 					{	psf_log_printf (psf, " %M : %d (too big, skipping)\n", marker, dword) ;
-						psf_binheader_readf (psf, "j", dword) ;
+						psf_binheader_readf (psf, "j", dword + (dword & 1)) ;
 						break ;
 						} ;
 
-					cptr = (char*) psf->buffer ;
-					psf_binheader_readf (psf, "b", cptr, dword) ;
-					cptr [dword - 1] = 0 ;
+					cptr = psf->u.cbuf ;
+					psf_binheader_readf (psf, "b", cptr, dword + (dword & 1)) ;
+					cptr [dword] = 0 ;
 
-					for (k = 0 ; k < dword ; k++)
+					for (k = 0 ; k < (int) dword ; k++)
 						if (! isprint (cptr [k]))
 						{	cptr [k] = 0 ;
 							break ;
@@ -514,35 +574,34 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 
 			case NAME_MARKER :
 					psf_binheader_readf (psf, "E4", &dword) ;
-					dword += (dword & 1) ;
 					if (dword == 0)
 						break ;
-					if (dword > SIGNED_SIZEOF (psf->buffer))
+					if (dword > SIGNED_SIZEOF (psf->u.scbuf) - 2)
 					{	psf_log_printf (psf, " %M : %d (too big)\n", marker, dword) ;
 						return SFE_INTERNAL ;
 						} ;
 
-					cptr = (char*) psf->buffer ;
-					psf_binheader_readf (psf, "b", cptr, dword) ;
-					cptr [dword - 1] = 0 ;
+					cptr = psf->u.cbuf ;
+					psf_binheader_readf (psf, "b", cptr, dword + (dword & 1)) ;
+					cptr [dword] = 0 ;
 					psf_log_printf (psf, " %M : %s\n", marker, cptr) ;
 					psf_store_string (psf, SF_STR_TITLE, cptr) ;
 					break ;
 
 			case ANNO_MARKER :
 					psf_binheader_readf (psf, "E4", &dword) ;
-					dword += (dword & 1) ;
 					if (dword == 0)
 						break ;
-					if (dword > SIGNED_SIZEOF (psf->buffer))
+					if (dword > SIGNED_SIZEOF (psf->u.scbuf) - 2)
 					{	psf_log_printf (psf, " %M : %d (too big)\n", marker, dword) ;
 						return SFE_INTERNAL ;
 						} ;
 
-					cptr = (char*) psf->buffer ;
-					psf_binheader_readf (psf, "b", cptr, dword) ;
-					cptr [dword - 1] = 0 ;
+					cptr = psf->u.cbuf ;
+					psf_binheader_readf (psf, "b", cptr, dword + (dword & 1)) ;
+					cptr [dword] = 0 ;
 					psf_log_printf (psf, " %M : %s\n", marker, cptr) ;
+					psf_store_string (psf, SF_STR_COMMENT, cptr) ;
 					break ;
 
 			case INST_MARKER :
@@ -556,15 +615,23 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 					{	unsigned char bytes [6] ;
 						short gain ;
 
+						if (psf->instrument == NULL && (psf->instrument = psf_instrument_alloc ()) == NULL)
+							return SFE_MALLOC_FAILED ;
+
 						psf_binheader_readf (psf, "b", bytes, 6) ;
-						psf_log_printf (psf, 	"  Base Note : %u\n  Detune    : %u\n"
-												"  Low  Note : %u\n  High Note : %u\n"
-												"  Low  Vel. : %u\n  High Vel. : %u\n",
-								bytes [0], bytes [1], bytes [2], bytes [3], bytes [4], bytes [5]) ;
-
+						psf_log_printf (psf, "  Base Note : %u\n  Detune    : %u\n"
+											"  Low  Note : %u\n  High Note : %u\n"
+											"  Low  Vel. : %u\n  High Vel. : %u\n",
+											bytes [0], bytes [1], bytes [2], bytes [3], bytes [4], bytes [5]) ;
+						psf->instrument->basenote = bytes [0] ;
+						psf->instrument->detune = bytes [1] ;
+						psf->instrument->key_lo = bytes [2] ;
+						psf->instrument->key_hi = bytes [3] ;
+						psf->instrument->velocity_lo = bytes [4] ;
+						psf->instrument->velocity_hi = bytes [5] ;
 						psf_binheader_readf (psf, "E2", &gain) ;
-						psf_log_printf (psf, 	"  Gain (dB) : %d\n", gain) ;
-
+						psf->instrument->gain = gain ;
+						psf_log_printf (psf, "  Gain (dB) : %d\n", gain) ;
 						} ;
 					{	short	mode ; /* 0 - no loop, 1 - forward looping, 2 - backward looping */
 						const char	*loop_mode ;
@@ -572,37 +639,78 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 
 						psf_binheader_readf (psf, "E222", &mode, &begin, &end) ;
 						loop_mode = get_loop_mode_str (mode) ;
-						psf_log_printf (psf, 	"  Sustain\n   mode  : %d => %s\n   begin : %u\n   end   : %u\n",
-								mode, loop_mode, begin, end) ;
+						mode = get_loop_mode (mode) ;
+						if (mode == SF_LOOP_NONE)
+						{	psf->instrument->loop_count = 0 ;
+							psf->instrument->loops [0].mode = SF_LOOP_NONE ;
+							}
+						else
+						{	psf->instrument->loop_count = 1 ;
+							psf->instrument->loops [0].mode = SF_LOOP_FORWARD ;
+							psf->instrument->loops [0].start = begin ;
+							psf->instrument->loops [0].end = end ;
+							psf->instrument->loops [0].count = 0 ;
+							} ;
+						psf_log_printf (psf, "  Sustain\n   mode  : %d => %s\n   begin : %u\n   end   : %u\n",
+											mode, loop_mode, begin, end) ;
 						psf_binheader_readf (psf, "E222", &mode, &begin, &end) ;
 						loop_mode = get_loop_mode_str (mode) ;
-						psf_log_printf (psf, 	"  Release\n   mode  : %d => %s\n   begin : %u\n   end   : %u\n",
-								mode, loop_mode, begin, end) ;
+						mode = get_loop_mode (mode) ;
+						if (mode == SF_LOOP_NONE)
+							psf->instrument->loops [0].mode = SF_LOOP_NONE ;
+						else
+						{	psf->instrument->loop_count += 1 ;
+							psf->instrument->loops [1].mode = SF_LOOP_FORWARD ;
+							psf->instrument->loops [1].start = begin ;
+							psf->instrument->loops [1].end = end ;
+							psf->instrument->loops [1].count = 0 ;
+							} ;
+						psf_log_printf (psf, "  Release\n   mode  : %d => %s\n   begin : %u\n   end   : %u\n",
+										mode, loop_mode, begin, end) ;
 						} ;
+					instr_found++ ;
+					break ;
+
+			case basc_MARKER :
+					psf_binheader_readf (psf, "E4", &dword) ;
+					psf_log_printf (psf, " basc : %u\n", dword) ;
+
+					if ((error = aiff_read_basc_chunk (psf, dword)))
+						return error ;
 					break ;
 
 			case MARK_MARKER :
 					psf_binheader_readf (psf, "E4", &dword) ;
 					psf_log_printf (psf, " %M : %d\n", marker, dword) ;
-					{	unsigned short	mark_count, mark_id ;
-						unsigned char	pstr_len ;
-						unsigned int	position ;
+					{	unsigned short mark_id, n = 0 ;
+						unsigned char pstr_len ;
+						unsigned int position ;
 
-						bytesread = psf_binheader_readf (psf, "E2", &mark_count) ;
+						bytesread = psf_binheader_readf (psf, "E2", &n) ;
+						mark_count = n ;
+						markstr = calloc (mark_count, sizeof (MARK_ID_POS)) ;
 						psf_log_printf (psf, "  Count : %d\n", mark_count) ;
 
-						while (mark_count && bytesread < dword)
+						for (n = 0 ; n < mark_count && bytesread < dword ; n++)
 						{	bytesread += psf_binheader_readf (psf, "E241", &mark_id, &position, &pstr_len) ;
 							psf_log_printf (psf, "   Mark ID  : %u\n   Position : %u\n", mark_id, position) ;
 
 							pstr_len += (pstr_len & 1) + 1 ; /* fudgy, fudgy, hack, hack */
 
-							bytesread += psf_binheader_readf (psf, "b", psf->buffer, pstr_len) ;
-							psf_log_printf (psf, "   Name     : %s\n", psf->buffer) ;
+							bytesread += psf_binheader_readf (psf, "b", psf->u.scbuf, pstr_len) ;
+							psf_log_printf (psf, "   Name     : %s\n", psf->u.scbuf) ;
 
-							mark_count -- ;
+							markstr [n].markerID = mark_id ;
+							markstr [n].position = position ;
+							/*
+							**	TODO if psf->u.scbuf is equal to
+							**	either Beg_loop, Beg loop or beg loop and spam
+							**	if (psf->instrument == NULL && (psf->instrument = psf_instrument_alloc ()) == NULL)
+							**		return SFE_MALLOC_FAILED ;
+							*/
 							} ;
 						} ;
+					mark_found++ ;
 					psf_binheader_readf (psf, "j", dword - bytesread) ;
 					break ;
 
@@ -625,7 +733,7 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 					if (isprint ((marker >> 24) & 0xFF) && isprint ((marker >> 16) & 0xFF)
 						&& isprint ((marker >> 8) & 0xFF) && isprint (marker & 0xFF))
 					{	psf_binheader_readf (psf, "E4", &dword) ;
-						psf_log_printf (psf, "%M : %d (unknown marker)\n", marker, dword) ;
+						psf_log_printf (psf, " %M : %d (unknown marker)\n", marker, dword) ;
 
 						psf_binheader_readf (psf, "j", dword) ;
 						break ;
@@ -646,11 +754,23 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 
 		if (psf_ftell (psf) >= psf->filelength - (2 * SIGNED_SIZEOF (dword)))
 			break ;
-
-		if (psf->logindex >= SIGNED_SIZEOF (psf->logbuffer) - 2)
-			return SFE_LOG_OVERRUN ;
 		} ; /* while (1) */
 
+	if (instr_found && mark_found)
+	{	int j ;
+
+		for (j = 0 ; j<psf->instrument->loop_count ; j ++)
+		{	if (j < ARRAY_LEN (psf->instrument->loops))
+			{	psf->instrument->loops [j].start = marker_to_position (markstr, psf->instrument->loops [j].start, mark_count) ;
+				psf->instrument->loops [j].end = marker_to_position (markstr, psf->instrument->loops [j].end, mark_count) ;
+				psf->instrument->loops [j].mode = SF_LOOP_FORWARD ;
+				} ;
+  			} ;
+		} ;
+
+	if (markstr)
+		free (markstr) ;
+
 	if (! (found_chunk & HAVE_FORM))
 		return SFE_AIFF_NO_FORM ;
 
@@ -667,7 +787,7 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 } /* aiff_read_header */
 
 static int
-aiff_close	(SF_PRIVATE *psf)
+aiff_close (SF_PRIVATE *psf)
 {
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
 	{	aiff_write_tailer (psf) ;
@@ -682,6 +802,8 @@ static int
 aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 {	int error = 0, bytesread, subformat ;
 
+	psf->u.scbuf [0] = 0 ;
+
 	bytesread = psf_binheader_readf (psf, "E4", &(comm_fmt->size)) ;
 
 	/* The COMM chunk has an int aligned to an odd word boundary. Some
@@ -703,11 +825,11 @@ aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 
 		bytesread += psf_binheader_readf (psf, "Em1", &(comm_fmt->encoding), &encoding_len) ;
 
-		memset (psf->buffer, 0, comm_fmt->size) ;
+		memset (psf->u.scbuf, 0, comm_fmt->size) ;
 
-		bytesread += psf_binheader_readf (psf, "b", psf->buffer,
+		bytesread += psf_binheader_readf (psf, "b", psf->u.scbuf,
 							comm_fmt->size - SIZEOF_AIFC_COMM + 1) ;
-		psf->buffer [encoding_len] = 0 ;
+		psf->u.scbuf [encoding_len] = 0 ;
 		} ;
 
 	psf_log_printf (psf, " COMM : %d\n", comm_fmt->size) ;
@@ -715,7 +837,7 @@ aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 	psf_log_printf (psf, "  Frames      : %u%s\n", comm_fmt->numSampleFrames, (comm_fmt->numSampleFrames == 0 && psf->filelength > 100) ? " (Should not be 0)" : "") ;
 	psf_log_printf (psf, "  Channels    : %d\n", comm_fmt->numChannels) ;
 
-	/*	Found some broken 'fl32' files with comm.samplesize == 16. Fix it here. */
+	/*      Found some broken 'fl32' files with comm.samplesize == 16. Fix it here. */
 
 	if ((comm_fmt->encoding == fl32_MARKER || comm_fmt->encoding == FL32_MARKER) && comm_fmt->sampleSize != 32)
 	{	psf_log_printf (psf, "  Sample Size : %d (should be 32)\n", comm_fmt->sampleSize) ;
@@ -807,19 +929,20 @@ aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 			error = SFE_UNIMPLEMENTED ;
 		} ;
 
-	if (! psf->buffer [0])
+	if (! psf->u.scbuf [0])
 		psf_log_printf (psf, "  Encoding    : %M\n", comm_fmt->encoding) ;
 	else
-		psf_log_printf (psf, "  Encoding    : %M => %s\n", comm_fmt->encoding, (char*) psf->buffer) ;
+		psf_log_printf (psf, "  Encoding    : %M => %s\n", comm_fmt->encoding, psf->u.scbuf) ;
 
 	return error ;
 } /* aiff_read_comm_chunk */
 
+
 static int
 aiff_write_header (SF_PRIVATE *psf, int calc_length)
 {	sf_count_t		current ;
-	unsigned char 	comm_sample_rate [10], comm_zero_bytes [2] = { 0, 0 } ;
-	unsigned int 	comm_type, comm_size, comm_encoding, comm_frames ;
+	unsigned char	comm_sample_rate [10], comm_zero_bytes [2] = { 0, 0 } ;
+	unsigned int	comm_type, comm_size, comm_encoding, comm_frames ;
 	int				k, endian ;
 	short			bit_width ;
 
@@ -839,7 +962,7 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 	if (psf->mode == SFM_RDWR && psf->dataoffset > 0)
 	{	/* Assuming here that the header has already been written and just
 		** needs to be corrected for new data length. That means that we
-		** only change the length fields of the FORM and SSND chunks;
+		** only change the length fields of the FORM and SSND chunks ;
 		** everything else can be skipped over.
 		*/
 
@@ -1008,6 +1131,7 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 	/* Write COMM chunk. */
 	psf_binheader_writef (psf, "Emm4", comm_type, COMM_MARKER, comm_size) ;
 
+	memset (comm_sample_rate, 0, sizeof (comm_sample_rate)) ;
 	uint2tenbytefloat (psf->sf.samplerate, comm_sample_rate) ;
 
 	psf_binheader_writef (psf, "Et242", psf->sf.channels, comm_frames, bit_width) ;
@@ -1017,15 +1141,70 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 	if (comm_type == AIFC_MARKER)
 		psf_binheader_writef (psf, "mb", comm_encoding, comm_zero_bytes, sizeof (comm_zero_bytes)) ;
 
+	if (psf->instrument != NULL)
+	{	MARK_ID_POS	m [4] ;
+		INST_CHUNK ch ;
+		unsigned short ct = 0 ;
+
+		memset (m, 0, sizeof (m)) ;
+
+		ch.baseNote = psf->instrument->basenote ;
+		ch.detune = psf->instrument->detune ;
+		ch.lowNote = psf->instrument->key_lo ;
+		ch.highNote = psf->instrument->key_hi ;
+		ch.lowVelocity = psf->instrument->velocity_lo ;
+		ch.highVelocity = psf->instrument->velocity_hi ;
+		ch.gain = psf->instrument->gain ;
+		if (psf->instrument->loops [0].mode != SF_LOOP_NONE)
+		{	ch.sustain_loop.playMode = 1 ;
+			ch.sustain_loop.beginLoop = ct ;
+			m [0].markerID = ct++ ;
+			m [0].position = psf->instrument->loops [0].start ;
+			ch.sustain_loop.endLoop = ct ;
+			m [1].markerID = ct++ ;
+			m [1].position = psf->instrument->loops [0].end ;
+			} ;
+		if (psf->instrument->loops [1].mode != SF_LOOP_NONE)
+		{	ch.release_loop.playMode = 1 ;
+			ch.release_loop.beginLoop = ct ;
+			m [2].markerID = ct++ ;
+			m [2].position = psf->instrument->loops [1].start ;
+			ch.release_loop.endLoop = ct ;
+			m [3].markerID = ct++ ;
+			m [3].position = psf->instrument->loops [1].end ;
+			}
+		else
+		{	ch.release_loop.playMode = 0 ;
+			ch.release_loop.beginLoop = 0 ;
+			ch.release_loop.endLoop = 0 ;
+			} ;
+		psf_binheader_writef (psf, "Em4b", INST_MARKER, sizeof (INST_CHUNK), &ch.baseNote, make_size_t (6)) ;
+		psf_binheader_writef (psf, "2222222", ch.gain, ch.sustain_loop.playMode,
+				ch.sustain_loop.beginLoop, ch.sustain_loop.endLoop, ch.release_loop.playMode,
+				ch.release_loop.beginLoop, ch.release_loop.endLoop) ;
+
+		if (ct == 2)
+			psf_binheader_writef (psf, "Em42241b241b",
+					MARK_MARKER, 2 * sizeof (MARK_ID_POS) + 16, 2,
+					m [0].markerID, m [0].position, 8, "beg loop", make_size_t (9),
+					m [1].markerID, m [1].position, 8, "end loop", make_size_t (9)) ;
+		else if (ct == 4)
+			psf_binheader_writef (psf, "Em42241b241b241b241b",
+					MARK_MARKER, 4 * sizeof (MARK_ID_POS) + 34, 4,
+					m [0].markerID, m [0].position, 8, "beg loop", make_size_t (9),
+					m [1].markerID, m [1].position, 8, "end loop", make_size_t (9),
+					m [2].markerID, m [2].position, 8, "beg loop", make_size_t (9),
+					m [3].markerID, m [3].position, 8, "end loop", make_size_t (9)) ;
+		} ;
+
 	if (psf->str_flags & SF_STR_LOCATE_START)
 		aiff_write_strings (psf, SF_STR_LOCATE_START) ;
 
-	if (psf->has_peak && psf->peak_loc == SF_PEAK_START)
-	{	psf_binheader_writef (psf, "Em4", PEAK_MARKER,
-			sizeof (PEAK_CHUNK) + psf->sf.channels * sizeof (PEAK_POS)) ;
+	if (psf->peak_info != NULL && psf->peak_info->peak_loc == SF_PEAK_START)
+	{	psf_binheader_writef (psf, "Em4", PEAK_MARKER, AIFF_PEAK_CHUNK_SIZE (psf->sf.channels)) ;
 		psf_binheader_writef (psf, "E44", 1, time (NULL)) ;
 		for (k = 0 ; k < psf->sf.channels ; k++)
-			psf_binheader_writef (psf, "Ef4", psf->pchunk->peaks [k].value, psf->pchunk->peaks [k].position) ; /* XXXXX */
+			psf_binheader_writef (psf, "Eft8", (float) psf->peak_info->peaks [k].value, psf->peak_info->peaks [k].position) ;
 		} ;
 
 	/* Write SSND chunk. */
@@ -1057,19 +1236,24 @@ aiff_write_tailer (SF_PRIVATE *psf)
 
 	psf->dataend = psf_fseek (psf, 0, SEEK_END) ;
 
-	if (psf->has_peak && psf->peak_loc == SF_PEAK_END)
-	{	psf_binheader_writef (psf, "Em4", PEAK_MARKER,
-			sizeof (PEAK_CHUNK) + psf->sf.channels * sizeof (PEAK_POS)) ;
+	/* Make sure tailer data starts at even byte offset. Pad if necessary. */
+	if (psf->dataend % 2 == 1)
+	{	psf_fwrite (psf->header, 1, 1, psf) ;
+		psf->dataend ++ ;
+		} ;
+
+	if (psf->peak_info != NULL && psf->peak_info->peak_loc == SF_PEAK_END)
+	{	psf_binheader_writef (psf, "Em4", PEAK_MARKER, AIFF_PEAK_CHUNK_SIZE (psf->sf.channels)) ;
 		psf_binheader_writef (psf, "E44", 1, time (NULL)) ;
 		for (k = 0 ; k < psf->sf.channels ; k++)
-			psf_binheader_writef (psf, "Ef4", psf->pchunk->peaks [k].value, psf->pchunk->peaks [k].position) ; /* XXXXX */
+			psf_binheader_writef (psf, "Eft8", (float) psf->peak_info->peaks [k].value, psf->peak_info->peaks [k].position) ;
 		} ;
 
 	if (psf->str_flags & SF_STR_LOCATE_END)
 		aiff_write_strings (psf, SF_STR_LOCATE_END) ;
 
 	/* Write the tailer. */
-	if (psf->headindex)
+	if (psf->headindex > 0)
 		psf_fwrite (psf->header, psf->headindex, 1, psf) ;
 
 	return 0 ;
@@ -1088,23 +1272,23 @@ aiff_write_strings (SF_PRIVATE *psf, int location)
 
 		switch (psf->strings [k].type)
 		{	case SF_STR_SOFTWARE :
-				psf_binheader_writef (psf, "Ems", APPL_MARKER, psf->strings [k].str) ;
+				psf_binheader_writef (psf, "EmS", APPL_MARKER, psf->strings [k].str) ;
 				break ;
 
 			case SF_STR_TITLE :
-				psf_binheader_writef (psf, "Ems", NAME_MARKER, psf->strings [k].str) ;
+				psf_binheader_writef (psf, "EmS", NAME_MARKER, psf->strings [k].str) ;
 				break ;
 
 			case SF_STR_COPYRIGHT :
-				psf_binheader_writef (psf, "Ems", c_MARKER, psf->strings [k].str) ;
+				psf_binheader_writef (psf, "EmS", c_MARKER, psf->strings [k].str) ;
 				break ;
 
 			case SF_STR_ARTIST :
-				psf_binheader_writef (psf, "Ems", AUTH_MARKER, psf->strings [k].str) ;
+				psf_binheader_writef (psf, "EmS", AUTH_MARKER, psf->strings [k].str) ;
 				break ;
 
 			case SF_STR_COMMENT :
-				psf_binheader_writef (psf, "Ems", COMT_MARKER, psf->strings [k].str) ;
+				psf_binheader_writef (psf, "EmS", ANNO_MARKER, psf->strings [k].str) ;
 				break ;
 
 			/*
@@ -1144,6 +1328,17 @@ get_loop_mode_str (short mode)
 	return "*** unknown" ;
 } /* get_loop_mode_str */
 
+static short
+get_loop_mode (short mode)
+{	switch (mode)
+	{	case 0 : return SF_LOOP_NONE ;
+		case 1 : return SF_LOOP_FORWARD ;
+		case 2 : return SF_LOOP_BACKWARD ;
+		} ;
+
+	return SF_LOOP_NONE ;
+} /* get_loop_mode */
+
 /*==========================================================================================
 **	Rough hack at converting from 80 bit IEEE float in AIFF header to an int and
 **	back again. It assumes that all sample rates are between 1 and 800MHz, which
@@ -1181,10 +1376,7 @@ tenbytefloat2int (unsigned char *bytes)
 static void
 uint2tenbytefloat (unsigned int num, unsigned char *bytes)
 {	unsigned int mask = 0x40000000 ;
-	int		count ;
-
-
-	memset (bytes, 0, 10) ;
+	int	count ;
 
 	if (num <= 1)
 	{	bytes [0] = 0x3F ;
@@ -1215,6 +1407,69 @@ uint2tenbytefloat (unsigned int num, unsigned char *bytes)
 
 } /* uint2tenbytefloat */
 
+static int
+aiff_read_basc_chunk (SF_PRIVATE * psf, int datasize)
+{	const char * type_str ;
+	basc_CHUNK bc ;
+
+	psf_binheader_readf (psf, "E442", &bc.version, &bc.numBeats, &bc.rootNote) ;
+	psf_binheader_readf (psf, "E222", &bc.scaleType, &bc.sigNumerator, &bc.sigDenominator) ;
+	psf_binheader_readf (psf, "E2j", &bc.loopType, datasize - sizeof (bc)) ;
+
+	psf_log_printf (psf, "  Version ? : %u\n  Num Beats : %u\n  Root Note : 0x%x\n",
+						bc.version, bc.numBeats, bc.rootNote) ;
+
+	switch (bc.scaleType)
+	{	case basc_SCALE_MINOR :
+				type_str = "MINOR" ;
+				break ;
+		case basc_SCALE_MAJOR :
+				type_str = "MAJOR" ;
+				break ;
+		case basc_SCALE_NEITHER :
+				type_str = "NEITHER" ;
+				break ;
+		case basc_SCALE_BOTH :
+				type_str = "BOTH" ;
+				break ;
+		default :
+				type_str = "!!WRONG!!" ;
+				break ;
+		} ;
+
+	psf_log_printf (psf, "  ScaleType : 0x%x (%s)\n", bc.scaleType, type_str) ;
+	psf_log_printf (psf, "  Time Sig  : %d/%d\n", bc.sigNumerator, bc.sigDenominator) ;
+
+	switch (bc.loopType)
+	{	case basc_TYPE_ONE_SHOT :
+				type_str = "One Shot" ;
+				break ;
+		case basc_TYPE_LOOP :
+				type_str = "Loop" ;
+				break ;
+		default:
+				type_str = "!!WRONG!!" ;
+				break ;
+		} ;
+
+	psf_log_printf (psf, "  Loop Type : 0x%x (%s)\n", bc.loopType, type_str) ;
+
+	if ((psf->loop_info = calloc (1, sizeof (SF_LOOP_INFO))) == NULL)
+		return SFE_MALLOC_FAILED ;
+
+	psf->loop_info->time_sig_num	= bc.sigNumerator ;
+	psf->loop_info->time_sig_den	= bc.sigDenominator ;
+	psf->loop_info->loop_mode		= (bc.loopType == basc_TYPE_ONE_SHOT) ? SF_LOOP_NONE : SF_LOOP_FORWARD ;
+	psf->loop_info->num_beats		= bc.numBeats ;
+
+	/* Can always be recalculated from other known fields. */
+	psf->loop_info->bpm = (1.0 / psf->sf.frames) * psf->sf.samplerate
+							* ((bc.numBeats * 4.0) / bc.sigDenominator) * 60.0 ;
+	psf->loop_info->root_key = bc.rootNote ;
+
+	return 0 ;
+} /* aiff_read_basc_chunk */
+
 /*
 ** Do not edit or modify anything in this comment block.
 ** The arch-tag line is a file identity tag for the GNU Arch
