@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2005 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2006 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -29,6 +29,8 @@
 
 #ifndef SNDFILE_H
 #include "sndfile.h"
+#elif HAVE_INTTYPES_H
+#include <inttypes.h>
 #endif
 
 #ifdef __cplusplus
@@ -52,10 +54,10 @@
 
 #define	SF_BUFFER_LEN			(8192*2)
 #define	SF_FILENAME_LEN			(512)
-#define	SF_HEADER_LEN			(4100)
-#define	SF_TEXT_LEN				(1024)
 #define SF_SYSERR_LEN			(256)
 #define SF_MAX_STRINGS			(16)
+#define SF_STR_BUFFER_LEN		(8192)
+#define	SF_HEADER_LEN			(4100 + SF_STR_BUFFER_LEN)
 
 #define	PSF_SEEK_ERROR			((sf_count_t) -1)
 
@@ -155,7 +157,7 @@ typedef struct
 	/*
 	** This is not ISO compliant C. It works on some compilers which
 	** don't support the ISO standard flexible struct array which is
-	** used above. If your compiler doesn't ike this I suggest you find
+	** used above. If your compiler doesn't like this I suggest you find
 	** youself a 1999 ISO C standards compilant compiler. GCC-3.X is
 	** highly recommended.
 	*/
@@ -174,6 +176,11 @@ typedef struct
 	char 	*str ;
 } STR_DATA ;
 
+static inline size_t
+make_size_t (int x)
+{	return (size_t) x ;
+} /* size_t_of_int */
+
 /*=======================================================================================
 **	SF_PRIVATE stuct - a pointer to this struct is passed back to the caller of the
 **	sf_open_XXXX functions. The caller however has no knowledge of the struct's
@@ -215,7 +222,7 @@ typedef struct sf_private_tag
 	** sound files.
 	*/
 	STR_DATA		strings [SF_MAX_STRINGS] ;
-	char			str_storage [SF_BUFFER_LEN] ;
+	char			str_storage [SF_STR_BUFFER_LEN] ;
 	char			*str_end ;
 	int				str_flags ;
 
@@ -266,6 +273,7 @@ typedef struct sf_private_tag
 
 	/* Loop Info */
 	SF_LOOP_INFO	*loop_info ;
+	SF_INSTRUMENT	*instrument ;
 
 	sf_count_t		filelength ;	/* Overall length of (embedded) file. */
 	sf_count_t		fileoffset ;	/* Offset in number of bytes from beginning of file. */
@@ -314,7 +322,13 @@ typedef struct sf_private_tag
 	sf_count_t		(*seek) 		(struct sf_private_tag*, int mode, sf_count_t samples_from_start) ;
 	int				(*write_header)	(struct sf_private_tag*, int calc_length) ;
 	int				(*command)		(struct sf_private_tag*, int command, void *data, int datasize) ;
-	int				(*close)		(struct sf_private_tag*) ;
+
+	/*
+	**	Separate close functions for the codec and the container.
+	**	The codec close function is always called first.
+	*/
+	int				(*codec_close)		(struct sf_private_tag*) ;
+	int				(*container_close)	(struct sf_private_tag*) ;
 
 	char			*format_desc ;
 
@@ -622,7 +636,6 @@ int psf_ferror (SF_PRIVATE *psf) ;
 
 int		aiff_open	(SF_PRIVATE *psf) ;
 int		au_open		(SF_PRIVATE *psf) ;
-int		au_nh_open	(SF_PRIVATE *psf) ;	/* Headerless version of AU. */
 int		avr_open	(SF_PRIVATE *psf) ;
 int		htk_open	(SF_PRIVATE *psf) ;
 int		ircam_open	(SF_PRIVATE *psf) ;
@@ -644,6 +657,7 @@ int		caf_open	(SF_PRIVATE *psf) ;
 
 /* In progress. Do not currently work. */
 
+int		mpeg_open	(SF_PRIVATE *psf) ;
 int		ogg_open	(SF_PRIVATE *psf) ;
 int		rx2_open	(SF_PRIVATE *psf) ;
 int		txw_open	(SF_PRIVATE *psf) ;
@@ -665,6 +679,7 @@ int		dwvw_init		(SF_PRIVATE *psf, int bitwidth) ;
 int		gsm610_init		(SF_PRIVATE *psf) ;
 int		vox_adpcm_init	(SF_PRIVATE *psf) ;
 int		flac_init		(SF_PRIVATE *psf) ;
+int		g72x_init 		(SF_PRIVATE * psf) ;
 
 int 	dither_init		(SF_PRIVATE *psf, int mode) ;
 
@@ -681,6 +696,8 @@ int		interleave_init (SF_PRIVATE *psf) ;
 
 void	*psf_memset (void *s, int c, sf_count_t n) ;
 
+SF_INSTRUMENT * psf_instrument_alloc (void) ;
+
 /*------------------------------------------------------------------------------------
 ** Here's how we fix systems which don't snprintf / vsnprintf.
 ** Systems without these functions should use the
@@ -704,8 +721,6 @@ int missing_vsnprintf (char *str, size_t n, const char *fmt, ...) ;
 #define	LSF_VSNPRINTF	missing_vsnprintf
 #endif
 
-#endif /* SNDFILE_COMMON_H */
-
 /*------------------------------------------------------------------------------------
 ** Extra commands for sf_command(). Not for public use yet.
 */
@@ -728,6 +743,9 @@ int sf_dither_int		(const SF_DITHER_INFO *dither, const int *in, int *out, int c
 int sf_dither_float		(const SF_DITHER_INFO *dither, const float *in, float *out, int count) ;
 int sf_dither_double	(const SF_DITHER_INFO *dither, const double *in, double *out, int count) ;
 #endif
+
+#endif /* SNDFILE_COMMON_H */
+
 /*
 ** Do not edit or modify anything in this comment block.
 ** The arch-tag line is a file identity tag for the GNU Arch
