@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2001-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,22 +21,44 @@
 ** stdout.
 */
 
-#include "config.h"
+#include "sfconfig.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef HAVE_UNISTD_H
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
+
+#if HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
 
 #include "utils.h"
 
+#if (OS_IS_WIN32)
+
+int
+main (void)
+{
+	puts ("    stdio_test : this test doesn't work on win32.") ;
+	return 0 ;
+} /* main */
+
+#else
+
+#ifndef WIFEXITED
+#define WIFEXITED(s) (((s) & 0xff) == 0)
+#endif
+#ifndef WEXITSTATUS
+#define WEXITSTATUS(s) (((s) & 0xff00) >> 8)
+#endif
+
+
 static size_t	file_length (const char *filename) ;
 static int		file_exists (const char *filename) ;
 static void		stdio_test (const char *filetype) ;
@@ -71,7 +93,7 @@ stdio_test (const char *filetype)
 
 	snprintf (buffer, sizeof (buffer), "./stdout_test %s > stdio.%s", filetype, filetype) ;
 	if ((retval = system (buffer)))
-	{	retval = WEXITSTATUS (retval) ;
+	{	retval = WIFEXITED (retval) ? WEXITSTATUS (retval) : 1 ;
 		printf ("%s : %s", buffer, (strerror (retval))) ;
 		exit (1) ;
 		} ;
@@ -84,14 +106,14 @@ stdio_test (const char *filetype)
 
 	snprintf (buffer, sizeof (buffer), "./stdin_test %s < stdio.%s", filetype, filetype) ;
 	if ((retval = system (buffer)))
-	{	retval = WEXITSTATUS (retval) ;
+	{	retval = WIFEXITED (retval) ? WEXITSTATUS (retval) : 1 ;
 		printf ("%s : %s", buffer, (strerror (retval))) ;
 		exit (1) ;
 		} ;
 
 	snprintf (buffer, sizeof (buffer), "rm stdio.%s", filetype) ;
 	if ((retval = system (buffer)))
-	{	retval = WEXITSTATUS (retval) ;
+	{	retval = WIFEXITED (retval) ? WEXITSTATUS (retval) : 1 ;
 		printf ("%s : %s", buffer, (strerror (retval))) ;
 		exit (1) ;
 		} ;
@@ -125,6 +147,9 @@ file_exists (const char *filename)
 
 	return 1 ;
 } /* file_exists */
+
+#endif
+
 /*
 ** Do not edit or modify anything in this comment block.
 ** The arch-tag line is a file identity tag for the GNU Arch 
